/* Copyright (c) 1997, MpegTV LLC */

#ifndef lint
static char rcsid[] = "$Header: /net/hopkins/var/home/tristan/mpeg/dec/RCS/ci_file.c,v 1.2 1997/08/21 11:51:26 tristan Exp $";
static char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/misc.h"

#include "mtvp/sii_api.h"

/*
 * This is a working example of SII plug-in.
 * See the README file.
 */
 
/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
/*
 * In this example, our stream structure contains only a file descriptor.
 * The STREAM structure can contain absolutely anything you want,  The nature of
 * this structure is only known to the SII routines defined in the SII plug-in.
 */
typedef struct {
	int		fd;
} STREAM;

/*--------------------------------------------------------------------------+
| _init																		|
+--------------------------------------------------------------------------*/
int
_init()
{
	/*
	 * If the library exports a routine named  _init(),  then  that
     * code  is  executed  before  dlopen()  returns.
     */

	printf("_init(): loading custom stream interface\n");
		
	return 0;
}


/*--------------------------------------------------------------------------+
| _fini																		|
+--------------------------------------------------------------------------*/
int
_fini()
{
	/*
	 * If the dynamic library exports a routine named _fini(),
	 * then that routine is called just before the library is unloaded.
	 */

	printf("_fini(): unloading custom stream interface\n");
		
	return 0;
}

/*--------------------------------------------------------------------------+
| SII_open																	|
+--------------------------------------------------------------------------*/
void *
SII_open(
	const void *data,	/* assumed to be a filename in this example */
	u_int		size	/* ignored in this example */
)
{
	const char *filename = (const char *)data;
	int		fd;
	STREAM	*p;

	/*
	 * The parameter of SII_open is an arbitrary chunk of data (and
	 * its size) that is passed by the application to the COM_SII_OPEN
	 * command.  The block of data identifies the stream to be opened.
	 * 
	 * SII_open returns a pointer to an opaque object, i.e.
	 * only your SII plug-in known about the structure of the
	 * opaque object.  If the specified stream cannot be opened,
	 * SII_open returns NULL and sets errno.
	 * 
	 * In all the following routines, the "void *stream" parameter
	 * is the pointer that was returned by SII_open.
	 */	

	assert(size == strlen(filename));
	
	printf("SII_open calls open(\"%s\", O_RDONLY)\n", filename);
	
	/* check that sii_feedback() works */
	sii_feedback(data, size);
	
	/* in this example, we just open the specified file */
	fd = open(filename, O_RDONLY);
	
	if (fd == -1) {
		perror("SII_open");
		return NULL;
	}

	/* allocate storage for our opaque STREAM structure */
	p = malloc(sizeof(STREAM));
	
	if (p == NULL) {
		perror("SII_open");
		close(fd);
		return NULL;
	}
	
	/* the file descriptor for the opened file is stored in our STREAM structure */
	p->fd = fd;
	
	/*
	 * Return a pointer to our STREAM structure.
	 * Note that SII_open is defined as returning a (void *) pointer,
	 * because the returned pointer is opaque, i.e. STREAM is unknown
	 * from the caller (mtvp).
	 */
	return (void *)p;
}

/*--------------------------------------------------------------------------+
| SII_fd																	|
+--------------------------------------------------------------------------*/
int
SII_fd(void *stream)
{
	STREAM *p = (STREAM *)stream;

	/*
	 * Returns the file descriptor associated with the stream.
	 * 
	 * This routine must return a real file descriptor.
	 * The Player calls a blocking select on this file descriptor
	 * and if there is something available for reading on it, calls SII_read
	 * (provided that there is buffer space available to read some input data).
	 * 
	 * After calling SII_fd, mtvp calls fcntl with F_SETFL
	 * to set O_NONBLOCK on the fd.
	 */

	printf("SII_fd returns %d\n", p->fd);
	
	return p->fd;
}


/*--------------------------------------------------------------------------+
| SII_close																	|
+--------------------------------------------------------------------------*/
int
SII_close(void *stream)
{
	STREAM *p = (STREAM *)stream;

	/*
	 * Close the stream.
	 * 
	 * This routine must close every fd opened by SII_open and free the STREAM
	 * structure.  All the resources allocated for the stream should be released.
	 * 
	 * After a call to SII_close, the "void *stream"
	 * should be considered invalid.
	 */

	printf("SII_close\n");
	
	close(p->fd);
	free(p);
	
	return 0;
}


/*--------------------------------------------------------------------------+
| SII_read																	|
+--------------------------------------------------------------------------*/
int
SII_read(
	void *stream,
	char *buf,
	size_t count
)
{
	STREAM *p = (STREAM *)stream;
	int		n;

	/*
	 * SII_read must behave exactly like the
	 * regular read routine when used on a fd
	 * that has O_NONBLOCK set.
	 * 
	 * If data becomes available for reading on fd, SII_read
	 * is called. Note that mtvp may call SII_read at any time, even
	 * when there is no data available.
	 * 
	 * SII_read should always behave like a non-blocking read
	 * (i.e. like read behaves when the flag O_NONBLOCK set).
	 * This means that SII_read should never block, even
	 * if there is not enough data available.
	 * 
	 * If there is no data available,
	 * SII_read should return -1 with errno set to EAGAIN.
	 * 
	 * If there is less data available than requested,
	 * SII_read should copy all the data available into the
	 * buffer and return the number of bytes copied.
	 * 
	 * SII_read returning 0 means EOF.
	 * 
	 * SII_read returning -1 with errno NOT set to EAGAIN
	 * means an error occured.  All read errors (except EAGAIN)
	 * are considered like EOF.
	 */

	n = read(p->fd, buf, count);
	
	printf("SII_read calls read(%d, buf, %d), returns %d\n", p->fd, count, n);
	
	return n;
}



/*--------------------------------------------------------------------------+
| SII_lseek																	|
+--------------------------------------------------------------------------*/
off_t
SII_lseek(
	void *stream,
	off_t offset,
	int whence
)
{
	STREAM *p = (STREAM *)stream;
	off_t	n;
	
	/*
	 * SII_lseek must behave exactly like the regular lseek routine.
	 * 
	 * In particular:
	 * 
	 * SII_lseek should return -1 with errno set to ESPIPE
	 * if the input does not support seeking.
	 */
	 
	n = lseek(p->fd, offset, whence);

	printf("SII_lseek calls lseek(%d, %ldL, %d), returns %ld\n", p->fd, (long)offset, whence, (long)n);
	
	return n;
}


/*--------------------------------------------------------------------------+
| SII_size																	|
+--------------------------------------------------------------------------*/
off_t
SII_size(
	void *stream
)
{
	STREAM *p = (STREAM *)stream;
	struct	stat stat_buf;
	off_t	size;
	int		n;

	/*
	 * SII_size, if defined, must return the size of the file, in bytes.
	 * 
	 * If SII_size is not defined, appropriate calls to SII_lseek are used
	 * to determine the size of the file.
	 */
	 
	n = fstat(p->fd, &stat_buf);

	printf("SII_size calls fstat(%d, ...), returns %d, size = %lu\n", p->fd, n, (u_long)stat_buf.st_size);
	
	if (n != 0) {
		return (off_t)-1;
	}
	
	return stat_buf.st_size;
}


/*--------------------------------------------------------------------------+
| SII_cntl																	|
+--------------------------------------------------------------------------*/
int
SII_cntl(
	void *stream,
	const void *data,
	u_int		size
)
{
	/*
	 * SII_cntl is called if and only if the application sends
	 * the message COM_SII_CNTL.  The parameter "data" is a pointer to
	 * an arbitrary chunk of data specified by the application.
	 * "size" is the size of the chunk of data.
	 */
	 
	/*
	 * In this example, the data block passed to SII_cntl() is known to be
	 * a string, so we just print it to show that we received it.
	 */
	 
	printf("SII_cntl: data = \"%s\", size = %u\n", (const char *)data, size);
	
	return 0;
}

/* eof */
